#!/usr/bin/env bash

# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# A helper script for TravisCI builds that saves the std
# out and err streams in a log file. This is needed
# because otherwise TravisCI complains that there is too
# much logging on stdout

ant -q test 2>&1 > ant-test.log
RC=$?
{
    if grep -q "Testsuites with failed tests:" ant-test.log; then
        for failed in $(awk 'failed == 1 { print $2 }; /Testsuites with failed tests:/ { failed=1 }' ant-test.log); do
            echo "Failed test: $failed"
            cat output/build/logs/$failed
            echo
        done
    fi
} > test-failures.log
exit $RC
